@@ -13,11 +13,13 @@
 
 #include <string>
 #include <map>
+#include <vector>
 #include "../srtcore/srt.h"
 #include "../srtcore/logging_api.h"
 
 srt_logging::LogLevel::type SrtParseLogLevel(std::string level);
 std::set<srt_logging::LogFA> SrtParseLogFA(std::string fa, std::set<std::string>* punknown = nullptr);
+void ParseLogFASpec(const std::vector<std::string>& speclist, std::string& w_on, std::string& w_off);
 const std::map<std::string, int> SrtLogFAList();
 
 SRT_API extern std::map<std::string, int> srt_level_names;
