@@ -45,7 +45,7 @@ typedef struct {
 typedef struct tag_hcrypt_Ctx {
         struct tag_hcrypt_Ctx * alt;    /* Alternative ctx (even/odd) */
 
-#define HCRYPT_CTX_F_MSG        0x00FF  /* Aligned wiht message header flags */		
+#define HCRYPT_CTX_F_MSG        0x00FF  /* Aligned with message header flags */
 #define HCRYPT_CTX_F_eSEK       HCRYPT_MSG_F_eSEK
 #define HCRYPT_CTX_F_oSEK       HCRYPT_MSG_F_oSEK
 #define HCRYPT_CTX_F_xSEK       HCRYPT_MSG_F_xSEK
@@ -68,6 +68,7 @@ typedef struct tag_hcrypt_Ctx {
 #define HCRYPT_CTX_MODE_AESECB  1   /* Electronic Code Book mode */
 #define HCRYPT_CTX_MODE_AESCTR  2   /* Counter mode */
 #define HCRYPT_CTX_MODE_AESCBC  3   /* Cipher-block chaining mode */
+#define HCRYPT_CTX_MODE_AESGCM  4   /* AES GCM authenticated encryption */
         unsigned         mode;
 
         struct {
