@@ -61,10 +61,12 @@ namespace srt_logging
 {
 extern Logger qrlog;
 extern Logger qslog;
+extern Logger tslog;
 }
 
 using srt_logging::qrlog;
 using srt_logging::qslog;
+using srt_logging::tslog;
 
 using namespace srt::sync;
 
@@ -504,7 +506,7 @@ srt::CRcvLossList::~CRcvLossList()
     delete[] m_caSeq;
 }
 
-void srt::CRcvLossList::insert(int32_t seqno1, int32_t seqno2)
+int srt::CRcvLossList::insert(int32_t seqno1, int32_t seqno2)
 {
     // Data to be inserted must be larger than all those in the list
     if (m_iLargestSeq != SRT_SEQNO_NONE && CSeqNo::seqcmp(seqno1, m_iLargestSeq) <= 0)
@@ -522,7 +524,7 @@ void srt::CRcvLossList::insert(int32_t seqno1, int32_t seqno2)
                  log << "RCV-LOSS/insert: (" << seqno1 << "," << seqno2
                      << ") to be inserted is too small: m_iLargestSeq=" << m_iLargestSeq << ", m_iLength=" << m_iLength
                      << ", m_iHead=" << m_iHead << ", m_iTail=" << m_iTail << " -- REJECTING");
-            return;
+            return 0;
         }
     }
     m_iLargestSeq = seqno2;
@@ -538,19 +540,19 @@ void srt::CRcvLossList::insert(int32_t seqno1, int32_t seqno2)
 
         m_caSeq[m_iHead].inext  = -1;
         m_caSeq[m_iHead].iprior = -1;
-        m_iLength += CSeqNo::seqlen(seqno1, seqno2);
-
-        return;
+        const int n = CSeqNo::seqlen(seqno1, seqno2);
+        m_iLength += n;
+        return n;
     }
 
     // otherwise searching for the position where the node should be
-    int offset = CSeqNo::seqoff(m_caSeq[m_iHead].seqstart, seqno1);
+    const int offset = CSeqNo::seqoff(m_caSeq[m_iHead].seqstart, seqno1);
     if (offset < 0)
     {
         LOGC(qrlog.Error,
              log << "RCV-LOSS/insert: IPE: new LOSS %(" << seqno1 << "-" << seqno2 << ") PREDATES HEAD %"
                  << m_caSeq[m_iHead].seqstart << " -- REJECTING");
-        return;
+        return -1;
     }
 
     int loc = (m_iHead + offset) % m_iSize;
@@ -575,7 +577,9 @@ void srt::CRcvLossList::insert(int32_t seqno1, int32_t seqno2)
         m_iTail                = loc;
     }
 
-    m_iLength += CSeqNo::seqlen(seqno1, seqno2);
+    const int n = CSeqNo::seqlen(seqno1, seqno2);
+    m_iLength += n;
+    return n;
 }
 
 bool srt::CRcvLossList::remove(int32_t seqno)
@@ -715,20 +719,43 @@ bool srt::CRcvLossList::remove(int32_t seqno)
 
 bool srt::CRcvLossList::remove(int32_t seqno1, int32_t seqno2)
 {
-    if (seqno1 <= seqno2)
+    if (CSeqNo::seqcmp(seqno1, seqno2) > 0)
     {
-        for (int32_t i = seqno1; i <= seqno2; ++i)
-            remove(i);
+        return false;
     }
-    else
+    for (int32_t i = seqno1; CSeqNo::seqcmp(i, seqno2) <= 0; i = CSeqNo::incseq(i))
     {
-        for (int32_t j = seqno1; j < CSeqNo::m_iMaxSeqNo; ++j)
-            remove(j);
-        for (int32_t k = 0; k <= seqno2; ++k)
-            remove(k);
+        remove(i);
+    }
+    return true;
+}
+
+int32_t srt::CRcvLossList::removeUpTo(int32_t seqno_last)
+{
+    int32_t first = getFirstLostSeq();
+    if (first == SRT_SEQNO_NONE)
+    {
+        //HLOGC(tslog.Debug, log << "rcv-loss: DROP to %" << seqno_last << " - empty list");
+        return first; // empty, so nothing to remove
     }
 
-    return true;
+    if (CSeqNo::seqcmp(seqno_last, first) < 0)
+    {
+        //HLOGC(tslog.Debug, log << "rcv-loss: DROP to %" << seqno_last << " - first %" << first << " is newer, exitting");
+        return first; // seqno_last older than first - nothing to remove
+    }
+
+    HLOGC(tslog.Debug, log << "rcv-loss: DROP to %" << seqno_last << " ...");
+
+    // NOTE: seqno_last is past-the-end here. Removed are only seqs
+    // that are earlier than this.
+    for (int32_t i = first; CSeqNo::seqcmp(i, seqno_last) <= 0; i = CSeqNo::incseq(i))
+    {
+        //HLOGC(tslog.Debug, log << "... removing %" << i);
+        remove(i);
+    }
+
+    return first;
 }
 
 bool srt::CRcvLossList::find(int32_t seqno1, int32_t seqno2) const
@@ -839,8 +866,10 @@ srt::CRcvFreshLoss::Emod srt::CRcvFreshLoss::revoke(int32_t lo, int32_t hi)
     // ITEM:  <lo, hi>                      <--- delete
     // If the sequence range is older than the range to be revoked,
     // delete it anyway.
-    if (CSeqNo::seqcmp(lo, seq[1]) > 0)
+    if (lo != SRT_SEQNO_NONE && CSeqNo::seqcmp(lo, seq[1]) > 0)
         return DELETE;
+    // IF <lo> is NONE, then rely simply on that item.hi <% arg.hi,
+    // which is a condition at the end.
 
     // LOHI:  <lo, hi>
     // ITEM:             <lo, hi>  <-- NOTFOUND
@@ -868,3 +897,53 @@ srt::CRcvFreshLoss::Emod srt::CRcvFreshLoss::revoke(int32_t lo, int32_t hi)
 
     return DELETE;
 }
+
+bool srt::CRcvFreshLoss::removeOne(std::deque<CRcvFreshLoss>& w_container, int32_t sequence, int* pw_had_ttl)
+{
+    for (size_t i = 0; i < w_container.size(); ++i)
+    {
+        const int had_ttl = w_container[i].ttl;
+        Emod wh = w_container[i].revoke(sequence);
+
+        if (wh == NONE)
+            continue;  // Not found. Search again.
+
+        if (wh == DELETE)   //  ... oo ... x ... o ... => ... oo ... o ...
+        {
+            // Removed the only element in the record - remove the record.
+            w_container.erase(w_container.begin() + i);
+        }
+        else if (wh == SPLIT) // ... ooxooo ... => ... oo ... ooo ...
+        {
+            // Create a new element that will hold the upper part of the range,
+            // and the found one modify to be the lower part of the range.
+
+            // Keep the current end-of-sequence value for the second element
+            int32_t next_end = w_container[i].seq[1];
+
+            // seq-1 set to the end of this element
+            w_container[i].seq[1] = CSeqNo::decseq(sequence);
+            // seq+1 set to the begin of the next element
+            int32_t next_begin = CSeqNo::incseq(sequence);
+
+            // Use position of the NEXT element because insertion happens BEFORE pointed element.
+            // Use the same TTL (will stay the same in the other one).
+            w_container.insert(w_container.begin() + i + 1,
+                    CRcvFreshLoss(next_begin, next_end, w_container[i].ttl));
+        }
+        // For STRIPPED:  ... xooo ... => ... ooo ...
+        // i.e. there's nothing to do.
+
+        // Every loss is unique. We're done here.
+        if (pw_had_ttl)
+            *pw_had_ttl = had_ttl;
+
+        return true;
+    }
+
+    if (pw_had_ttl)
+        *pw_had_ttl = 0;
+    return false;
+
+}
+
