@@ -61,17 +61,20 @@ modified by
 #include "packet.h"
 #include "udt.h"
 
+namespace srt
+{
+
 namespace ACKWindowTools
 {
    struct Seq
    {
        int32_t iACKSeqNo;                                   // Seq. No. of the ACK packet
        int32_t iACK;                                        // Data packet Seq. No. carried by the ACK packet
-       srt::sync::steady_clock::time_point tsTimeStamp;     // The timestamp when the ACK was sent
+       sync::steady_clock::time_point tsTimeStamp;     // The timestamp when the ACK was sent
    };
 
    void store(Seq* r_aSeq, const size_t size, int& r_iHead, int& r_iTail, int32_t seq, int32_t ack);
-   int acknowledge(Seq* r_aSeq, const size_t size, int& r_iHead, int& r_iTail, int32_t seq, int32_t& r_ack, const srt::sync::steady_clock::time_point& currtime);
+   int acknowledge(Seq* r_aSeq, const size_t size, int& r_iHead, int& r_iTail, int32_t seq, int32_t& r_ack, const sync::steady_clock::time_point& currtime);
 }
 
 template <size_t SIZE>
@@ -104,7 +107,7 @@ public:
       /// @param [in] currtime The timestamp of ACKACK packet reception by the receiver
       /// @return RTT
 
-   int acknowledge(int32_t seq, int32_t& r_ack, const srt::sync::steady_clock::time_point& currtime)
+   int acknowledge(int32_t seq, int32_t& r_ack, const sync::steady_clock::time_point& currtime)
    {
        return ACKWindowTools::acknowledge(m_aSeq, SIZE, m_iHead, m_iTail, seq, r_ack, currtime);
    }
@@ -145,14 +148,14 @@ public:
         m_iProbeWindowPtr(0),
         m_iLastSentTime(0),
         m_iMinPktSndInt(1000000),
-        m_tsLastArrTime(srt::sync::steady_clock::now()),
+        m_tsLastArrTime(sync::steady_clock::now()),
         m_tsCurrArrTime(),
         m_tsProbeTime(),
         m_Probe1Sequence(SRT_SEQNO_NONE)
     {
         // Exception: up to CUDT ctor
-        srt::sync::setupMutex(m_lockPktWindow, "PktWindow");
-        srt::sync::setupMutex(m_lockProbeWindow, "ProbeWindow");
+        sync::setupMutex(m_lockPktWindow, "PktWindow");
+        sync::setupMutex(m_lockProbeWindow, "ProbeWindow");
         CPktTimeWindowTools::initializeWindowArrays(m_aPktWindow, m_aProbeWindow, m_aBytesWindow, ASIZE, PSIZE);
     }
 
@@ -172,7 +175,7 @@ public:
    int getPktRcvSpeed(int& w_bytesps) const
    {
        // Lock access to the packet Window
-       srt::sync::ScopedLock cg(m_lockPktWindow);
+       sync::ScopedLock cg(m_lockPktWindow);
 
        int pktReplica[ASIZE];          // packet information window (inter-packet time)
        return getPktRcvSpeed_in(m_aPktWindow, pktReplica, m_aBytesWindow, ASIZE, (w_bytesps));
@@ -190,7 +193,7 @@ public:
    int getBandwidth() const
    {
        // Lock access to the packet Window
-       srt::sync::ScopedLock cg(m_lockProbeWindow);
+       sync::ScopedLock cg(m_lockProbeWindow);
 
        int probeReplica[PSIZE];
        return getBandwidth_in(m_aProbeWindow, probeReplica, PSIZE);
@@ -213,12 +216,12 @@ public:
 
    void onPktArrival(int pktsz = 0)
    {
-       srt::sync::ScopedLock cg(m_lockPktWindow);
+       sync::ScopedLock cg(m_lockPktWindow);
 
-       m_tsCurrArrTime = srt::sync::steady_clock::now();
+       m_tsCurrArrTime = sync::steady_clock::now();
 
        // record the packet interval between the current and the last one
-       m_aPktWindow[m_iPktWindowPtr] = (int) srt::sync::count_microseconds(m_tsCurrArrTime - m_tsLastArrTime);
+       m_aPktWindow[m_iPktWindowPtr] = (int) sync::count_microseconds(m_tsCurrArrTime - m_tsLastArrTime);
        m_aBytesWindow[m_iPktWindowPtr] = pktsz;
 
        // the window is logically circular
@@ -231,7 +234,7 @@ public:
    }
 
    /// Shortcut to test a packet for possible probe 1 or 2
-   void probeArrival(const srt::CPacket& pkt, bool unordered)
+   void probeArrival(const CPacket& pkt, bool unordered)
    {
        const int inorder16 = pkt.m_iSeqNo & PUMASK_SEQNO_PROBE;
 
@@ -252,7 +255,7 @@ public:
    }
 
    /// Record the arrival time of the first probing packet.
-   void probe1Arrival(const srt::CPacket& pkt, bool unordered)
+   void probe1Arrival(const CPacket& pkt, bool unordered)
    {
        if (unordered && pkt.m_iSeqNo == m_Probe1Sequence)
        {
@@ -263,13 +266,13 @@ public:
            return;
        }
 
-       m_tsProbeTime = srt::sync::steady_clock::now();
+       m_tsProbeTime = sync::steady_clock::now();
        m_Probe1Sequence = pkt.m_iSeqNo; // Record the sequence where 16th packet probe was taken
    }
 
    /// Record the arrival time of the second probing packet and the interval between packet pairs.
 
-   void probe2Arrival(const srt::CPacket& pkt)
+   void probe2Arrival(const CPacket& pkt)
    {
        // Reject probes that don't refer to the very next packet
        // towards the one that was lately notified by probe1Arrival.
@@ -285,10 +288,10 @@ public:
        // Grab the current time before trying to acquire
        // a mutex. This might add extra delay and therefore
        // screw up the measurement.
-       const srt::sync::steady_clock::time_point now = srt::sync::steady_clock::now();
+       const sync::steady_clock::time_point now = sync::steady_clock::now();
 
        // Lock access to the packet Window
-       srt::sync::ScopedLock cg(m_lockProbeWindow);
+       sync::ScopedLock cg(m_lockProbeWindow);
 
        m_tsCurrArrTime = now;
 
@@ -298,8 +301,8 @@ public:
 
        // record the probing packets interval
        // Adjust the time for what a complete packet would have take
-       const int64_t timediff = srt::sync::count_microseconds(m_tsCurrArrTime - m_tsProbeTime);
-       const int64_t timediff_times_pl_size = timediff * srt::CPacket::SRT_MAX_PAYLOAD_SIZE;
+       const int64_t timediff = sync::count_microseconds(m_tsCurrArrTime - m_tsProbeTime);
+       const int64_t timediff_times_pl_size = timediff * CPacket::SRT_MAX_PAYLOAD_SIZE;
 
        // Let's take it simpler than it is coded here:
        // (stating that a packet has never zero size)
@@ -325,27 +328,28 @@ public:
    }
 
 private:
-   int m_aPktWindow[ASIZE];                                 // Packet information window (inter-packet time)
+   int m_aPktWindow[ASIZE];                            // Packet information window (inter-packet time)
    int m_aBytesWindow[ASIZE];
-   int m_iPktWindowPtr;                                     // Position pointer of the packet info. window
-   mutable srt::sync::Mutex m_lockPktWindow;                // Used to synchronize access to the packet window
+   int m_iPktWindowPtr;                                // Position pointer of the packet info. window
+   mutable sync::Mutex m_lockPktWindow;                // Used to synchronize access to the packet window
 
-   int m_aProbeWindow[PSIZE];                               // Record inter-packet time for probing packet pairs
-   int m_iProbeWindowPtr;                                   // Position pointer to the probing window
-   mutable srt::sync::Mutex m_lockProbeWindow;              // Used to synchronize access to the probe window
+   int m_aProbeWindow[PSIZE];                          // Record inter-packet time for probing packet pairs
+   int m_iProbeWindowPtr;                              // Position pointer to the probing window
+   mutable sync::Mutex m_lockProbeWindow;              // Used to synchronize access to the probe window
 
-   int m_iLastSentTime;                                     // Last packet sending time
-   int m_iMinPktSndInt;                                     // Minimum packet sending interval
+   int m_iLastSentTime;                                // Last packet sending time
+   int m_iMinPktSndInt;                                // Minimum packet sending interval
 
-   srt::sync::steady_clock::time_point m_tsLastArrTime;     // Last packet arrival time
-   srt::sync::steady_clock::time_point m_tsCurrArrTime;     // Current packet arrival time
-   srt::sync::steady_clock::time_point m_tsProbeTime;       // Arrival time of the first probing packet
-   int32_t m_Probe1Sequence;                                // Sequence number for which the arrival time was notified
+   sync::steady_clock::time_point m_tsLastArrTime;     // Last packet arrival time
+   sync::steady_clock::time_point m_tsCurrArrTime;     // Current packet arrival time
+   sync::steady_clock::time_point m_tsProbeTime;       // Arrival time of the first probing packet
+   int32_t m_Probe1Sequence;                           // Sequence number for which the arrival time was notified
 
 private:
    CPktTimeWindow(const CPktTimeWindow&);
    CPktTimeWindow &operator=(const CPktTimeWindow&);
 };
 
+} // namespace srt
 
 #endif
