@@ -4,6 +4,7 @@
 #include <thread>
 #include <condition_variable>
 #include "gtest/gtest.h"
+#include "test_env.h"
 #include "api.h"
 #include "epoll.h"
 
@@ -14,8 +15,7 @@ using namespace srt;
 
 TEST(CEPoll, InfiniteWait)
 {
-    ASSERT_EQ(srt_startup(), 0);
-
+    srt::TestInit srtinit;
     const int epoll_id = srt_epoll_create();
     ASSERT_GE(epoll_id, 0);
 
@@ -25,13 +25,11 @@ TEST(CEPoll, InfiniteWait)
         0, 0, 0, 0), SRT_ERROR);
 
     EXPECT_EQ(srt_epoll_release(epoll_id), 0);
-
-    EXPECT_EQ(srt_cleanup(), 0);
 }
 
 TEST(CEPoll, WaitNoSocketsInEpoll)
 {
-    ASSERT_EQ(srt_startup(), 0);
+    srt::TestInit srtinit;
 
     const int epoll_id = srt_epoll_create();
     ASSERT_GE(epoll_id, 0);
@@ -47,12 +45,11 @@ TEST(CEPoll, WaitNoSocketsInEpoll)
 
     EXPECT_EQ(srt_epoll_release(epoll_id), 0);
 
-    EXPECT_EQ(srt_cleanup(), 0);
 }
 
 TEST(CEPoll, WaitNoSocketsInEpoll2)
 {
-    ASSERT_EQ(srt_startup(), 0);
+    srt::TestInit srtinit;
 
     const int epoll_id = srt_epoll_create();
     ASSERT_GE(epoll_id, 0);
@@ -63,12 +60,11 @@ TEST(CEPoll, WaitNoSocketsInEpoll2)
 
     EXPECT_EQ(srt_epoll_release(epoll_id), 0);
 
-    EXPECT_EQ(srt_cleanup(), 0);
 }
 
 TEST(CEPoll, WaitEmptyCall)
 {
-    ASSERT_EQ(srt_startup(), 0);
+    srt::TestInit srtinit;
 
     SRTSOCKET client_sock = srt_create_socket();
     ASSERT_NE(client_sock, SRT_ERROR);
@@ -87,12 +83,11 @@ TEST(CEPoll, WaitEmptyCall)
                 -1, 0, 0, 0, 0), SRT_ERROR);
 
     EXPECT_EQ(srt_epoll_release(epoll_id), 0);
-    EXPECT_EQ(srt_cleanup(), 0);
 }
 
 TEST(CEPoll, UWaitEmptyCall)
 {
-    ASSERT_EQ(srt_startup(), 0);
+    srt::TestInit srtinit;
 
     SRTSOCKET client_sock = srt_create_socket();
     ASSERT_NE(client_sock, SRT_ERROR);
@@ -111,12 +106,11 @@ TEST(CEPoll, UWaitEmptyCall)
 
     EXPECT_EQ(srt_epoll_release(epoll_id), 0);
 
-    EXPECT_EQ(srt_cleanup(), 0);
 }
 
 TEST(CEPoll, WaitAllSocketsInEpollReleased)
 {
-    ASSERT_EQ(srt_startup(), 0);
+    srt::TestInit srtinit;
 
     SRTSOCKET client_sock = srt_create_socket();
     ASSERT_NE(client_sock, SRT_ERROR);
@@ -146,12 +140,11 @@ TEST(CEPoll, WaitAllSocketsInEpollReleased)
 
     EXPECT_EQ(srt_epoll_release(epoll_id), 0);
 
-    EXPECT_EQ(srt_cleanup(), 0);
 }
 
 TEST(CEPoll, WaitAllSocketsInEpollReleased2)
 {
-    ASSERT_EQ(srt_startup(), 0);
+    srt::TestInit srtinit;
 
     SRTSOCKET client_sock = srt_create_socket();
     ASSERT_NE(client_sock, SRT_ERROR);
@@ -176,12 +169,11 @@ TEST(CEPoll, WaitAllSocketsInEpollReleased2)
 
     EXPECT_EQ(srt_epoll_release(epoll_id), 0);
 
-    EXPECT_EQ(srt_cleanup(), 0);
 }
 
 TEST(CEPoll, WrongEpoll_idOnAddUSock)
 {
-    ASSERT_EQ(srt_startup(), 0);
+    srt::TestInit srtinit;
 
     SRTSOCKET client_sock = srt_create_socket();
     ASSERT_NE(client_sock, SRT_ERROR);
@@ -199,13 +191,12 @@ TEST(CEPoll, WrongEpoll_idOnAddUSock)
 
     EXPECT_EQ(srt_epoll_release(epoll_id), 0);
 
-    EXPECT_EQ(srt_cleanup(), 0);
 }
 
 
 TEST(CEPoll, HandleEpollEvent)
 {
-    ASSERT_EQ(srt_startup(), 0);
+    srt::TestInit srtinit;
 
     SRTSOCKET client_sock = srt_create_socket();
     EXPECT_NE(client_sock, SRT_ERROR);
@@ -256,7 +247,6 @@ TEST(CEPoll, HandleEpollEvent)
         throw;
     }
 
-    EXPECT_EQ(srt_cleanup(), 0);
 }
 
 
@@ -266,7 +256,7 @@ TEST(CEPoll, HandleEpollEvent)
 // be notified about connection break via polling the accepted socket.
 TEST(CEPoll, NotifyConnectionBreak)
 {
-    ASSERT_EQ(srt_startup(), 0);
+    srt::TestInit srtinit;
 
     // 1. Prepare client
     SRTSOCKET client_sock = srt_create_socket();
@@ -376,13 +366,12 @@ TEST(CEPoll, NotifyConnectionBreak)
     if (!state_valid)
         cerr << "socket state: " << state << endl;
 
-    EXPECT_EQ(srt_cleanup(), 0);
 }
 
 
 TEST(CEPoll, HandleEpollEvent2)
 {
-    ASSERT_EQ(srt_startup(), 0);
+    srt::TestInit srtinit;
 
     SRTSOCKET client_sock = srt_create_socket();
     EXPECT_NE(client_sock, SRT_ERROR);
@@ -438,13 +427,12 @@ TEST(CEPoll, HandleEpollEvent2)
         throw;
     }
 
-    EXPECT_EQ(srt_cleanup(), 0);
 }
 
 
 TEST(CEPoll, HandleEpollNoEvent)
 {
-    ASSERT_EQ(srt_startup(), 0);
+    srt::TestInit srtinit;
 
     SRTSOCKET client_sock = srt_create_socket();
     EXPECT_NE(client_sock, SRT_ERROR);
@@ -490,12 +478,11 @@ TEST(CEPoll, HandleEpollNoEvent)
         throw;
     }
 
-    EXPECT_EQ(srt_cleanup(), 0);
 }
 
 TEST(CEPoll, ThreadedUpdate)
 {
-    ASSERT_EQ(srt_startup(), 0);
+    srt::TestInit srtinit;
 
     SRTSOCKET client_sock = srt_create_socket();
     EXPECT_NE(client_sock, SRT_ERROR);
@@ -556,13 +543,10 @@ TEST(CEPoll, ThreadedUpdate)
         cerr << ex.getErrorMessage() << endl;
         throw;
     }
-
-
-    EXPECT_EQ(srt_cleanup(), 0);
 }
 
 
-class TestEPoll: public testing::Test
+class TestEPoll: public srt::Test
 {
 protected:
 
@@ -692,7 +676,7 @@ protected:
 
             ASSERT_EQ(rlen, 1); // get exactly one read event without writes
             ASSERT_EQ(wlen, 0); // get exactly one read event without writes
-            ASSERT_EQ(read[0], servsock); // read event is for bind socket    	
+            ASSERT_EQ(read[0], servsock); // read event is for bind socket
         }
 
         sockaddr_in scl;
@@ -750,10 +734,8 @@ protected:
         srt_close(servsock);
     }
 
-    void SetUp() override
+    void setup() override
     {
-        ASSERT_EQ(srt_startup(), 0);
-
         m_client_pollid = srt_epoll_create();
         ASSERT_NE(SRT_ERROR, m_client_pollid);
 
@@ -762,11 +744,10 @@ protected:
 
     }
 
-    void TearDown() override
+    void teardown() override
     {
         (void)srt_epoll_release(m_client_pollid);
         (void)srt_epoll_release(m_server_pollid);
-        srt_cleanup();
     }
 };
 
